#pragma once

#include "vbuffer.h"
#include <functional>
#include <list>
#include <memory>
#include <thread>

class EDID;
class VScreen;

class VCard
{
public:
    explicit VCard(VScreen *screen = nullptr);
    ~VCard();
    void connect(const uint32_t pixel_area_limit, const uint32_t pixel_per_second_limit);
    void disconnect();
    bool is_connected() const;
    void handle_events(evdi_event_context *context);

    std::function<void(struct evdi_mode, void *user_data)> m_modeHandler;
    std::function<void(std::shared_ptr<VBuffer> buffer, void *user_data)> acquire_framebuffer_cb;
    std::function<void(struct evdi_cursor_set, void *user_data)> m_cursor_set_cb;
    std::function<void(struct evdi_cursor_move, void *user_data)> m_cursor_move_cb;

    struct evdi_mode getMode() const;
    void enable_cursor_events(bool enable = true) const;
    int get_event_source() const;

    void run(bool create_thread = false);
    void wait_events(int waiting_time);
    void take_control(VScreen *screen);

protected:
    void setMode(struct evdi_mode mode);
    void makeBuffers(int count);
    void clearBuffers();
    void request_update();
    void grab_pixels();

    // friend void default_dpms_handler(int dpms_mode, void *user_data);
    friend void default_mode_changed_handler(struct evdi_mode mode, void *user_data);
    friend void default_update_ready_handler(int buffer_to_be_updated, void *user_data);
    // friend void default_crtc_state_handler(int state, void *user_data);
    friend void default_cursor_set_handler(struct evdi_cursor_set cursor_set, void *user_data);
    friend void default_cursor_move_handler(struct evdi_cursor_move cursor_move, void *user_data);
    // friend void default_ddcci_data_handler(struct evdi_ddcci_data ddcci_data, void *user_data);

private:
    void loop();
    void setup_epoll();
    void cleanup();
    void process_events();

private:
    int epoll_fd;
    evdi_handle evdiHandle;
    std::shared_ptr<evdi_event_context> eventContext;
    evdi_mode mode;
    EDID *edid;
    VScreen *m_screen;

    std::list<std::shared_ptr<VBuffer>> buffers;
    std::shared_ptr<VBuffer> buffer_requested;
    std::thread m_thread;
    bool m_running;
};
