#pragma once

#include <evdi_lib.h>
#include <map>
#include <memory>

class VCard;
class VBuffer;
class VScreen
{
public:
    VScreen(bool separateCursorEvents = false);
    ~VScreen();
    void update();

    int event_source() const;
    void handle_events();
    void enable_cursor_events() const;
    void take_control(VScreen *screen, bool create_thread = true);
    bool is_connected() const;

protected:
    virtual void on_cursor_set(evdi_cursor_set cursor_set);
    virtual void on_cursor_move(evdi_cursor_move cursor_move);
    virtual void on_buffer_update(std::shared_ptr<VBuffer> buffer);
    virtual void on_mode_change(evdi_mode mode);

private:
    virtual void init();
    static void cursor_set_handler(evdi_cursor_set cursor_set, void *user_data);
    static void cursor_move_handler(evdi_cursor_move cursor_move, void *user_data);
    static void buffer_update_handler(std::shared_ptr<VBuffer> buffer, void *user_data);
    static void mode_handler(evdi_mode mode, void *user_data);

    std::shared_ptr<VCard> m_card;
    evdi_event_context context;
};

typedef std::shared_ptr<VScreen> ScreenPtr;
